////////////////////////////////////////////////////////////
//
// File: main.cpp
// Author: Sven Goossens
// Purpose:
//      A skeleton for the final assignment
//      Without adaptations, this should draw 2 pixels on the screen
//
///////////////////////////////////////////////////////////
#include <stdio.h>
#include <stdlib.h>
#include "glut.h"			 // glut header file. Never include glut before stdlib.h!
#include <string.h>

#define _USE_MATH_DEFINES	// Signal math.h that we would like defines like M_PI
#include <math.h>			// Might come in usefull for cosine functions and stuff like that

#include "main.h"			// Function declarations and some definitions
#include "dlist.h"			// base class of the drawtools
#include "drawtools.h"      // contains all you need to draw stuff

//    Global variables
// None! (at the moment)

/*
      void init(void)
      This function is called when glut is initialized. Use it to initialize
      variables and setup things that have to be done BEFORE the actual
      picture is drawn.
*/
void init(void)
{
	glutInitDisplayMode (GLUT_DOUBLE | GLUT_RGB);	// Double buffering, RGB format
	glutInitWindowSize(WIDTH,HEIGHT);				// 1024 x 500 pixel window
	glutInitWindowPosition(0,0);					// Window appear in the top-left corner of the screen
	glutCreateWindow("Final Task: Skeleton");		// Window title
	glutDisplayFunc(display);						// Main display function
	glutReshapeFunc(reshape);						// Reshape function, called when the window resizes

	// You might want to remove these after you are done testing with them
	glutTimerFunc(1000, alarm, 112); // Example Timer functions
	glutTimerFunc(1500, alarm, 1);
	glutTimerFunc(2000, alarm, 2);

	/*
		Set the background color. The color is in the Red/Green/BLue/Alpha
		format, ranging from 0.0 to 1.0.
		For example glClearColor(1.0, 0.0, 0.0, 1.0); delivers a red
		background.
	*/

	glClearColor(1.0, 1.0, 1.0, 1.0);   // white background
	/*
		The last call to glColor3f determines the color with which the
		next object will be drawn. RGB format.
	*/
	glColor3f(1.0, 0.0, 0.0);           //draw in red

	/*
		The following funtions setup the view of the screen.
		OpenGL actually creates a 3D world, which is displayed trough a
		viewport(the glut window). The way pixels are projected onto
		this viewport is setup here.
		Keep them like this if you don't need fancy options
	*/
	glMatrixMode(GL_PROJECTION);        // Next lines will adjust the projection matrix
	glLoadIdentity();                   // Reset matrix, returns viewport to origin
											// and undo possible rotations

	// Set the amount of points on the drawing grid
	// Also determines the viewport: we are looing at the rectangle between
	// (0,0) and (WIDTH,HEIGHT)
	gluOrtho2D(0.0, (float)WIDTH, 0.0, (float)HEIGHT);

	glMatrixMode(GL_MODELVIEW);

}
//---------------------------------------------------------------------------
/*
	void alarm(int alarmnumber)
	Demonstration of a timer callback. alarmnumber is used to distinguish
	between different timers.
*/
void alarm(int alarmnumber) {
	if(alarmnumber == 112)
	{
		printf("Ring Ring!!! This was alarm 1!\n");
		printf("Alarm reset, will ring again in 100 ms.\n");

		// register timer again with a value != 112, so it won't 
		// trigger the printf
		glutTimerFunc(100, alarm, 1337);
	}
	else
	{
		printf("Ring Ring!!! Alarm with alarmnumber %d called!\n",
									alarmnumber);
	}
}
//---------------------------------------------------------------------------
//	reshape: Handle window resizing (reshaping) events
//	The reshape callback is called when the window is resized or moved
//	In the current implementation, the drawing grid is extended or truncated 
//	when the window is reshaped
void reshape(int w, int h) 
{	
	glViewport(0.0f, 0.0f, (GLfloat)w, (GLfloat)h);
	glMatrixMode(GL_PROJECTION);  
	glLoadIdentity();
	gluOrtho2D(0.0, (GLdouble)w, 0.0, (GLdouble)h);
}
//---------------------------------------------------------------------------
/*
	void display(void)
	The main display callback. This callback is called when the screen has
	to be redrawn. This is when:
	-The screen first becomes active
	-The screen is resized
	-The screen is repositioned
*/
void display(void)
{
	GLfloat p[2];                   //Point coordinates are stored in a GLfloat

	glClear(GL_COLOR_BUFFER_BIT);  // clear the backbuffer
	glBegin(GL_POINTS);            // Start a new 'drawing' block.
				// This block is dedicated to drawing points(pixels)
		/* Draw points here
			The point (0,0) corresponds to the lower left corner.
			The following 4 lines will draw the point (100, 200) in red
		*/
		glColor3f(1.0, 0.0, 0.0);
		p[0] = 100.0;      // x coord
		p[1] = 200.0;      // y coord
		glVertex2fv(p);    // draw it

		glColor3f(0.0, 0.0, 1.0);
		p[0] = 200.0;      // x coord
		p[1] = 400.0;      // y coord
		glVertex2fv(p);    // draw it
	glEnd();        // End of the drawing block

	glFlush();			// Execute all commands waiting to be executed
	glutSwapBuffers();	// Swap the backbuffer and frontbuffer,
						// display what has been drawn
 }
//---------------------------------------------------------------------------
int main(int argc, char* argv[])
{
	// standard GLUT initialization 
	glutInit(&argc,argv);
	init();						// Call init function
	printf("starting Glut Loop...\n");
	glutMainLoop();				// Start drawing proceidure
	return 0;
}
//---------------------------------------------------------------------------